# Interaction IDS
CREATE_NEW_WHATIS_ID = "whatis-create-new"
CREATE_NEW_WHATIS_ID_SUBMIT = "whatis-create-new"
UPDATE_WHATIS_ID = "whatis-update"
UPDATE_WHATIS_ID_SUBMIT = "whatis-update"
DELETE_WHATIS_ID = "whatis-delete"
ROLLBACK_WHATIS_ID = "whatis-rollback"
WHATIS_ALL_ID = "whatis-all"
WHATIS_SEND_CHANNEL_ID = "whatis-send-channel"

# Whatis visible fields
WHATIS_FIELDS = [
    "terminology",
    "definition",
    "notes",
    "links",
    "point_of_contact",
    "added_by",
]
